using System;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Collections;

namespace DcartesClassLibrary
{
    public abstract class DataBase
    {
        /// <summary>
        /// ConnectionString recuperada do arquivo de configurao de aplicaes Web.config
        /// </summary>
        public static readonly string CONN_STRING = System.Configuration.ConfigurationSettings.AppSettings["ConnString"].ToString();
        
        /// <summary>
        /// Hashtable para armazenamento e manipulao dos parmetros das stored procedures da camada
        /// de negcio
        /// </summary>
        private static Hashtable parmCache = Hashtable.Synchronized(new Hashtable());

        /// <summary>
        /// Mtodo responsvel pela execuo de operaes sem retorno de resultados (INSERT, UPDATE e DELETE) em bancos de dados 
        /// </summary>
        /// <param name="cmdType">Tipo de comando (CommandType.StoredProcedure, CommandType.TableDirect, CommandType.Text) a ser
        /// executado pelo mtodo</param>
        /// <param name="cmdText">Comando SQL a ser executado, podendo ser uma string SQL convencional ou a referncia  uma
        /// stored procedure</param>
        /// <param name="cmdParms">Array de parmetros SQL contendo eventuais parmetros da instruo SQL</param>
        /// <returns>Nmero de registros afetados pela instruo SQL</returns>
        public static int ExecuteNonQuery(CommandType cmdType, string cmdText, params SqlParameter[] cmdParms)
        {

            SqlCommand cmd = new SqlCommand();

            using (SqlConnection conn = new SqlConnection(CONN_STRING))
            {
                try
                {
                    PrepareCommand(cmd, conn, null, cmdType, cmdText, cmdParms);
                    int val = cmd.ExecuteNonQuery();
                    cmd.Parameters.Clear();
                    return val;
                }
                catch (Exception ex)
                {
                    conn.Close();
                    conn.Dispose();
                    throw (ex);
                }
            }
        }

        /// <summary>
        /// Neste mtodo as operaes so executadas na forma de Transaction SQL, ou seja, execuo
        /// de mais de uma operao em bancos de dados
        /// </summary>
        /// <param name="trans">Instncia do objeto SqlTransaction do .NET Framework</param>
        /// <param name="cmdType">Tipo de comando (CommandType.StoredProcedure, CommandType.TableDirect, CommandType.Text) a ser
        /// executado pelo mtodo</param>
        /// <param name="cmdText">Comando SQL a ser executado, podendo ser uma string SQL convencional ou a referncia  uma
        /// stored procedure</param>
        /// <param name="cmd">SqlCommand contendo os parmetros</param>
        /// <returns>SqlCommand utilizado na execuo da instruo T-SQL</returns>
        public static SqlCommand ExecuteNonQueryCmd(SqlTransaction trans, CommandType cmdType, string cmdText, SqlCommand cmd)
        {
            PrepareCommand(cmd, trans.Connection, trans, cmdType, cmdText);
            cmd.ExecuteNonQuery();
            return cmd;
        }

        /// <summary>
        /// Neste OverLoad do mtodo as operaes so executadas na forma de Transaction SQL, ou seja, execuo
        /// de mais de uma operao em bancos de dados
        /// </summary>
        /// <param name="trans">Instncia do objeto SqlTransaction do .NET Framework</param>
        /// <param name="cmdType">Tipo de comando (CommandType.StoredProcedure, CommandType.TableDirect, CommandType.Text) a ser
        /// executado pelo mtodo</param>
        /// <param name="cmdText">Comando SQL a ser executado, podendo ser uma string SQL convencional ou a referncia  uma
        /// stored procedure</param>
        /// <param name="cmdParms">Array de parmetros SQL contendo eventuais parmetros da instruo SQL</param>
        /// <returns>Nmero de registros afetados pela instruo SQL</returns>
        public static int ExecuteNonQuery(SqlTransaction trans, CommandType cmdType, string cmdText, params SqlParameter[] cmdParms)
        {
            SqlCommand cmd = new SqlCommand();
            PrepareCommand(cmd, trans.Connection, trans, cmdType, cmdText, cmdParms);
            int val = cmd.ExecuteNonQuery();
            cmd.Parameters.Clear();
            return val;
        }

        /// <summary>
        /// Neste OverLoad do mtodo pode-se utilizar a mesma conexo utilizada em outras consultas anteriores ou
        /// posteriores. AVISO: Altamente recomendvel utilizar este overload com objetos
        /// SqlConnection inseridos no contexto de clusula "using" (responsvel pela execuo do Dispose das instncias declaradas).
        /// Objetos SqlConnection utilizados neste contexto no permanecem abertos ao final da execuo das instrues.
        /// </summary>
        /// <param name="conn">Instncia do objeto SqlConnection do .NET Framework</param>
        /// <param name="cmdType">Tipo de comando (CommandType.StoredProcedure, CommandType.TableDirect, CommandType.Text) a ser
        /// executado pelo mtodo</param>
        /// <param name="cmdText">Comando SQL a ser executado, podendo ser uma string SQL convencional ou a referncia  uma
        /// stored procedure</param>
        /// <param name="cmdParms">Array de parmetros SQL contendo eventuais parmetros da instruo SQL</param>
        /// <returns>Nmero de registros afetados pela instruo SQL</returns>
        public static int ExecuteNonQuery(SqlConnection conn, CommandType cmdType, string cmdText, params SqlParameter[] cmdParms)
        {
            SqlCommand cmd = new SqlCommand();
            PrepareCommand(cmd, conn, null, cmdType, cmdText, cmdParms);
            int val = cmd.ExecuteNonQuery();
            cmd.Parameters.Clear();
            return val;
        }

        public static int ExecuteNonQuery(CommandType cmdType, string cmdText, SqlCommand cmd)
        {
            using (SqlConnection conn = new SqlConnection(CONN_STRING))
            {
                try
                {
                    PrepareCommand(cmd, conn, null, cmdType, cmdText);
                    int val = cmd.ExecuteNonQuery();
                    cmd.Parameters.Clear();
                    return val;
                }
                catch (Exception ex)
                {
                    throw (ex);
                }
            }

        }

        /// <summary>
        /// Neste OverLoad do mtodo as operaes so executadas na forma de Transaction SQL, ou seja, execuo
        /// de mais de uma operao em bancos de dados
        /// </summary>
        /// <param name="trans">Instncia do objeto SqlTransaction do .NET Framework</param>
        /// <param name="cmdType">Tipo de comando (CommandType.StoredProcedure, CommandType.TableDirect, CommandType.Text) a ser
        /// executado pelo mtodo</param>
        /// <param name="cmdText">Comando SQL a ser executado, podendo ser uma string SQL convencional ou a referncia  uma
        /// stored procedure</param>
        /// <param name="cmdParms">Array de parmetros SQL contendo eventuais parmetros da instruo SQL</param>
        /// <returns>SqlCommand utilizado na execuo da instruo T-SQL</returns>
        public static SqlCommand ExecuteNonQueryCmd(SqlTransaction trans, CommandType cmdType, string cmdText, params SqlParameter[] cmdParms)
        {
            SqlCommand cmd = new SqlCommand();
            PrepareCommand(cmd, trans.Connection, trans, cmdType, cmdText, cmdParms);
            int val = cmd.ExecuteNonQuery();

            return cmd;
        }

        /// <summary>
        /// Mtodo responsvel pela execuo de operaes com retorno de resultados (instrues SELECT) atravs do 
        /// objeto SqlDataReader do ADO.NET 
        /// </summary>
        /// <param name="cmdType">Tipo de comando (CommandType.StoredProcedure, CommandType.TableDirect, CommandType.Text) a ser
        /// executado pelo mtodo</param>
        /// <param name="cmdText">Comando SQL a ser executado, podendo ser uma string SQL convencional ou a referncia a uma
        /// stored procedure</param>
        /// <param name="cmdParms">Array de parmetros SQL contendo eventuais parmetros da instruo SQL</param>
        /// <returns>SqlDataReader armazenando os registros selecionados</returns>
        public static SqlDataReader ExecuteReader(CommandType cmdType, string cmdText, params SqlParameter[] cmdParms)
        {
            SqlCommand cmd = new SqlCommand();
            SqlConnection conn = new SqlConnection(CONN_STRING);
            //-----------Tirando carcts especiais--------
            PreparaReader(cmdParms);
            //------------------------------------------
            try
            {
                PrepareCommand(cmd, conn, null, cmdType, cmdText, cmdParms);
                SqlDataReader rdr = cmd.ExecuteReader(CommandBehavior.CloseConnection);
                //cmd.Parameters.Clear();
                return rdr;
            }
            catch (Exception ex)
            {
                conn.Close();
                conn.Dispose();
                throw (ex);
            }

        }

        /// <summary>
        /// Mtodo responsvel pela execuo de operaes com retorno de resultados (instrues SELECT) atravs do 
        /// objeto SqlDataReader do ADO.NET 
        /// </summary>
        /// <param name="cmdType">Tipo de comando (CommandType.StoredProcedure, CommandType.TableDirect, CommandType.Text) a ser
        /// executado pelo mtodo</param>
        /// <param name="cmdText">Comando SQL a ser executado, podendo ser uma string SQL convencional ou a referncia a uma
        /// stored procedure</param>
        /// <param name="cmdParms">Array de parmetros SQL contendo eventuais parmetros da instruo SQL</param>
        /// <returns>SqlDataReader armazenando os registros selecionados</returns>
        public static SqlDataReader ExecuteReader(SqlTransaction trans, CommandType cmdType, string cmdText, params SqlParameter[] cmdParms)
        {
            SqlCommand cmd = new SqlCommand();
            //-----------Tirando carcts especiais--------
            PreparaReader(cmdParms);
            //------------------------------------------
            try
            {
                PrepareCommand(cmd, trans.Connection, trans, cmdType, cmdText, cmdParms);
                SqlDataReader rdr = cmd.ExecuteReader();
                //cmd.Parameters.Clear();
                return rdr;
            }
            catch (Exception ex)
            {

                throw (ex);
            }

        }

        /// <summary>
        /// Mtodo responsvel pela execuo de operaes com retorno de resultados (instrues SELECT) atravs do 
        /// objeto SqlDataReader do ADO.NET 
        /// </summary>
        /// <param name="cmdType">Tipo de comando (CommandType.StoredProcedure, CommandType.TableDirect, CommandType.Text) a ser
        /// executado pelo mtodo</param>
        /// <param name="cmdText">Comando SQL a ser executado, podendo ser uma string SQL convencional ou a referncia a uma
        /// stored procedure</param>
        /// <param name="cmdParms">Array de parmetros SQL contendo eventuais parmetros da instruo SQL</param>
        /// <returns>SqlDataReader armazenando os registros selecionados</returns>
        //public static SqlDataReader ExecuteReader(CommandType cmdType, string cmdText, SqlCommand cmd)
        //{
        //    SqlConnection conn = new SqlConnection(CONN_STRING);
        //    SqlDataReader rdr;
        //    if (cmd == null)
        //        cmd = new SqlCommand();

        //    try
        //    {
        //        PrepareCommand(cmd, conn, null, cmdType, cmdText);
        //        rdr = cmd.ExecuteReader(CommandBehavior.CloseConnection);
        //        cmd.Parameters.Clear();
        //    }
        //    catch (Exception ex)
        //    {
        //        conn.Dispose();
        //        throw (ex);
        //    }

        //    return rdr;
        //}

        /// <summary>
        /// Mtodo responsvel pela execuo de operaes com retorno de resultados (instrues SELECT) atravs do 
        /// objeto SqlDataReader do ADO.NET. Este overload do mtodo pode ser executado diversas vezes utilizando-se
        /// uma mesma instncia do objeto SqlConnection. AVISO: Altamente recomendvel utilizar este overload com objetos
        /// SqlConnection inseridos no contexto de clusula "using" (responsvel pela execuo do Dispose das instncias declaradas).
        /// Objetos SqlConnection utilizados neste contexto no permanecem abertos ao final da execuo das instrues.
        /// </summary>
        /// <param name="connection">Conexo</param>
        /// <param name="cmdType">Tipo de comando (CommandType.StoredProcedure, CommandType.TableDirect, CommandType.Text) a ser
        /// executado pelo mtodo</param>
        /// <param name="cmdText">Comando SQL a ser executado, podendo ser uma string SQL convencional ou a referncia a uma
        /// stored procedure</param>
        /// <param name="cmdParms">Array de parmetros SQL contendo eventuais parmetros da instruo SQL</param>
        /// <returns>SqlDataReader armazenando os registros selecionados</returns>
        public static SqlDataReader ExecuteReader(SqlConnection conn, CommandType cmdType, string cmdText, params SqlParameter[] cmdParms)
        {
            SqlCommand cmd = new SqlCommand();
            //-----------Tirando carcts especiais--------
            PreparaReader(cmdParms);
            //------------------------------------------
            try
            {
                PrepareCommand(cmd, conn, null, cmdType, cmdText, cmdParms);
                SqlDataReader rdr = cmd.ExecuteReader();
                cmd.Parameters.Clear();
                return rdr;
            }
            catch (Exception ex)
            {
                conn.Close();
                conn.Dispose();
                throw (ex);
            }
        }

        public static SqlDataReader ExecuteReaderCmd(SqlCommand cmd, CommandType cmdType, string cmdText, params SqlParameter[] cmdParms)
        {
            SqlConnection conn = new SqlConnection(CONN_STRING);

            try
            {
                PrepareCommand(cmd, conn, null, cmdType, cmdText, cmdParms);
                SqlDataReader rdr = cmd.ExecuteReader(CommandBehavior.CloseConnection);
                return rdr;
            }
            catch (Exception ex)
            {
                conn.Close();
                conn.Dispose();
                throw (ex);
            }
        }

        /// <summary>
        /// Mtodo responsvel pela execuo de operaes com retorno de resultados (instrues SELECT)
        /// atravs do objeto DataSet do ADO.NET 
        /// </summary>
        /// <param name="cmdText">Comando SQL a ser executado (string SQL convencional)</param>
        /// <returns>DataSet armazenando os registros selecionados</returns>
        public static DataSet ExecuteReaderDs(string cmdText)
        {
            SqlConnection conn = new SqlConnection(CONN_STRING);
            SqlDataAdapter da = new SqlDataAdapter(cmdText, conn);

            DataSet ds = new DataSet();

            try
            {
                da.Fill(ds);
            }
            catch (Exception ex)
            {
                conn.Close();
                conn.Dispose();
                throw (ex);
            }

            conn.Close();
            conn.Dispose();

            return ds;
        }

        /// <summary>
        /// Atravs deste OverLoad do mtodo pode-se preencher um DataSet (objeto do ADO.NET) atravs do processamento
        /// de instrues SQL convencionais ou procedimentos armazenados (stored procedures) existentes no SGDBR.
        /// </summary>
        /// <param name="cmdType">Tipo de comando (CommandType.StoredProcedure, CommandType.TableDirect, CommandType.Text) a ser
        /// executado pelo mtodo</param>
        /// <param name="cmdText">Comando SQL a ser executado, podendo ser uma string SQL convencional ou a referncia a uma
        /// stored procedure</param>
        /// <param name="cmdParms">Array de parmetros SQL contendo eventuais parmetros da instruo SQL</param>
        /// <returns>DataSet armazenando os registros selecionados</returns>
        public static DataSet ExecuteReaderDs(CommandType cmdType, string cmdText, params SqlParameter[] cmdParms)
        {
            SqlCommand cmd = new SqlCommand();
            SqlConnection conn = new SqlConnection(CONN_STRING);
            DataSet ds = new DataSet();
            //-----------Tirando carcts especiais--------
            PreparaReader(cmdParms);
            //------------------------------------------
            try
            {
                PrepareCommand(cmd, conn, null, cmdType, cmdText, cmdParms);
                SqlDataAdapter da = new SqlDataAdapter(cmd);
                da.Fill(ds);
                cmd.Parameters.Clear();

            }
            catch (Exception ex)
            {
                conn.Close();
                conn.Dispose();
                throw (ex);
            }
            conn.Close();
            conn.Dispose();
            return ds;
        }

        //overload c/ transaction
        public static DataSet ExecuteReaderDs(SqlTransaction trans, CommandType cmdType, string cmdText, params SqlParameter[] cmdParms)
        {
            SqlCommand cmd = new SqlCommand();
            //			SqlConnection conn = new SqlConnection(CONN_STRING);
            DataSet ds = new DataSet();
            //-----------Tirando carcts especiais--------
            PreparaReader(cmdParms);
            //------------------------------------------
            try
            {
                PrepareCommand(cmd, trans.Connection, trans, cmdType, cmdText, cmdParms);
                SqlDataAdapter da = new SqlDataAdapter(cmd);
                da.Fill(ds);
                cmd.Parameters.Clear();

            }
            catch (Exception ex)
            {
                //				conn.Close();
                //				conn.Dispose();
                throw (ex);
            }
            //			conn.Close();
            //			conn.Dispose();
            return ds;
        }

        /// <summary>
        /// Atravs deste mtodo pode-se preencher um DataSet (objeto do ADO.NET) atravs do processamento
        /// de instrues SQL convencionais ou procedimentos armazenados (stored procedures) existentes no SGDBR. Atravs
        /// do fornecimento do DataSet como parmetro pode-se preench-lo com mais de uma DataTable.
        /// </summary>		
        /// <param name="dsCompleto">DataSet a ser preenchido (podendo ser preenchido com mais de uma tabela depois). Passado como referncia.</param>
        /// <param name="nomeTabela">Nome da tabela a ser preenchida no DataSet</param>
        /// <param name="cmdType">Tipo de comando (CommandType.StoredProcedure, CommandType.TableDirect, CommandType.Text) a ser
        /// executado pelo mtodo</param>
        /// <param name="cmdText">Comando SQL a ser executado, podendo ser uma string SQL convencional ou a referncia a uma
        /// stored procedure</param>
        /// <param name="cmdParms">Array de parmetros SQL contendo eventuais parmetros da instruo SQL</param>
        public static void ExecuteReaderDsRef(ref DataSet dsCompleto, string nomeTabela,
            CommandType cmdType, string cmdText, params SqlParameter[] cmdParms)
        {
            SqlCommand cmd = new SqlCommand();
            SqlConnection conn = new SqlConnection(CONN_STRING);

            //-----------Tirando carcts especiais--------
            PreparaReader(cmdParms);
            //------------------------------------------
            try
            {
                PrepareCommand(cmd, conn, null, cmdType, cmdText, cmdParms);
                SqlDataAdapter da = new SqlDataAdapter(cmd);
                da.Fill(dsCompleto, nomeTabela);
                cmd.Parameters.Clear();

            }
            catch (Exception ex)
            {
                conn.Close();
                conn.Dispose();
                throw (ex);
            }
        }

        /// <summary>
        /// Mtodo responsvel pela recuperao de valores individuais (nicos) do banco de dados. Como retorno tem-se um objeto
        /// do tipo object que deve ser convertido em um momento posterior.
        /// </summary>
        /// <param name="cmdType">Tipo de comando (CommandType.StoredProcedure, CommandType.TableDirect, CommandType.Text) a ser
        /// executado pelo mtodo</param>
        /// <param name="cmdText">Comando SQL a ser executado, podendo ser uma string SQL convencional ou a referncia  uma
        /// stored procedure</param>
        /// <param name="cmdParms">Array de parmetros SQL contendo eventuais parmetros da instruo SQL</param>
        /// <returns>Objeto (valor) nico resultante da execuo da consulta no banco de dados</returns>
        public static object ExecuteScalar(CommandType cmdType, string cmdText, params SqlParameter[] cmdParms)
        {
            SqlCommand cmd = new SqlCommand();
            SqlConnection conn = new SqlConnection(CONN_STRING);

            try
            {
                PrepareCommand(cmd, conn, null, cmdType, cmdText, cmdParms);
                object obj = cmd.ExecuteScalar();
                cmd.Parameters.Clear();
                return obj;
            }
            catch (Exception ex)
            {
                throw (ex);
            }
            finally
            {
                conn.Close();
                conn.Dispose();
            }
        }

        /// <summary>
        /// Mtodo responsvel pela recuperao de valores individuais (nicos) do banco de dados. Como retorno tem-se um objeto
        /// do tipo object que deve ser convertido em um momento posterior. Este overload do mtodo deve ser utilizado para chamadas
        /// em contextos transacionais.
        /// </summary>
        /// <param name="trans">Instncia do objeto SqlTransaction do .NET Framework</param>
        /// <param name="cmdType">Tipo de comando (CommandType.StoredProcedure, CommandType.TableDirect, CommandType.Text) a ser
        /// executado pelo mtodo</param>
        /// <param name="cmdText">Comando SQL a ser executado, podendo ser uma string SQL convencional ou a referncia  uma
        /// stored procedure</param>
        /// <param name="cmdParms">Array de parmetros SQL contendo eventuais parmetros da instruo SQL</param>
        /// <returns>Objeto (valor) nico resultante da execuo da consulta no banco de dados</returns>
        public static object ExecuteScalar(SqlTransaction trans, CommandType cmdType, string cmdText, params SqlParameter[] cmdParms)
        {
            SqlCommand cmd = new SqlCommand();
            PrepareCommand(cmd, trans.Connection, trans, cmdType, cmdText, cmdParms);
            object obj = cmd.ExecuteScalar();
            cmd.Parameters.Clear();
            return obj;
        }

        /// <summary>
        /// Mtodo responsvel pela atribuio de uma referncia aos parmetros SQL armazenados na hashtable parmCache
        /// </summary>
        /// <param name="cacheKey">Nome, referncia ou chave dos valores armazenados na hashtable</param>
        /// <param name="cmdParms">Array de parmetros SQL contendo parmetros de instrues SQL</param>
        public static void CacheParameters(string cacheKey, params SqlParameter[] cmdParms)
        {
            parmCache[cacheKey] = cmdParms;
        }

        /// <summary>
        /// Mtodo responsvel pela recuperao do array de parmetros SQL armazenado na hashtable parmCache
        /// </summary>
        /// <param name="cacheKey">Nome, referncia ou chave dos valores armazenados na hashtable</param>
        /// <returns>Array de parmetros SQL contendo parmetros de instrues SQL</returns>
        public static SqlParameter[] GetCachedParameters(string cacheKey)
        {
            SqlParameter[] cachedParms = (SqlParameter[])parmCache[cacheKey];

            if (cachedParms == null)
                return null;

            SqlParameter[] clonedParms = new SqlParameter[cachedParms.Length];

            for (int i = 0, j = cachedParms.Length; i < j; i++)
                clonedParms[i] = (SqlParameter)((ICloneable)cachedParms[i]).Clone();

            return clonedParms;
        }

        /// <summary>
        /// Mtodo responsvel pela manipulao de instncias do objeto SqlCommand do ADO.NET
        /// para a execuo de operaes em bancos de dados
        /// </summary>
        /// <param name="cmd">Instncia do objeto SqlCommand a ser ajustado</param>
        /// <param name="conn">Instncia do objeto SqlConnection da conexo adotada</param>
        /// <param name="trans">Instncia do objeto SqlTransaction do .NET Framework</param>
        /// <param name="cmdType">Tipo de comando (CommandType.StoredProcedure, CommandType.TableDirect, CommandType.Text) a ser
        /// executado</param>
        /// <param name="cmdText">Comando SQL a ser executado, podendo ser uma string SQL convencional ou a referncia a uma
        /// stored procedure</param>
        /// <param name="cmdParms">Array de parmetros SQL contendo eventuais parmetros de instrues SQL</param>
        private static void PrepareCommand(SqlCommand cmd, SqlConnection conn, SqlTransaction trans, CommandType cmdType, string cmdText, SqlParameter[] cmdParms)
        {
            if (conn.State != ConnectionState.Open)
                conn.Open();

            cmd.Connection = conn;
            cmd.CommandText = cmdText;

            if (trans != null)
                cmd.Transaction = trans;

            cmd.CommandType = cmdType;

            if (cmdParms != null)
            {
                foreach (SqlParameter parm in cmdParms)
                {
                    if (cmd.Parameters.Contains(parm))
                        cmd.Parameters[parm.ParameterName] = parm;
                    else
                        cmd.Parameters.Add(parm);
                }		
            }
        }

        protected static void PrepareCommand(SqlCommand cmd, SqlConnection conn, SqlTransaction trans, CommandType cmdType, string cmdText)
        {
            if (conn.State != ConnectionState.Open)
                conn.Open();

            cmd.Connection = conn;
            cmd.CommandText = cmdText;

            if (trans != null)
                cmd.Transaction = trans;

            cmd.CommandType = cmdType;
        }

        private static void PreparaReader(SqlParameter[] cmdParms)
        {
            if (cmdParms != null)
            {
                foreach (SqlParameter parm in cmdParms)
                {
                    //----------------Retirando caracteres especiais-------
                    if ((parm.DbType.Equals(DbType.AnsiString) || parm.DbType.Equals(DbType.AnsiStringFixedLength)) && parm.Value != DBNull.Value)
                    {
                        parm.Value = RemoveQuote(parm.Value.ToString());
                        if (parm.Value.ToString() == "")
                            parm.Value = DBNull.Value;
                    }
                    //--------------------------------------------------
                }
            }
        }
        private static string RemoveQuote(string text)
        {
            int indice1 = text.IndexOf("'");
            int indice2 = text.IndexOf("`");
            int indice3 = text.IndexOf("\"");
            //string[] subStr;
            if (text.StartsWith("'") || text.StartsWith("\"") || text.StartsWith("`"))
                if (text.Length > 1)
                    RemoveQuote(text.Substring(1));
                else
                    return "";

            char[] separator1 = { '\'' };
            char[] separator2 = { '"' };
            char[] separator3 = { '`' };
            //Retirando '
            if (indice1 >= 0)
            {
                return text.Substring(0, indice1);
                //				if(text.Length-1>indice1 ) //se o apstrofo NO est na ltima posic
                //				{
                //					subStr=text.Split(separator1);
                //					text = subStr[0] + subStr[1];
                //				}
                //				else
                //				{
                //					text = Regex.Replace(text,"\'","");
                //				}
                //				RemoveQuote(text);
            }
            //Retirando `
            if (indice2 >= 0)
            {
                return text.Substring(0, indice2);
                //				if(text.Length-1>indice2 ) //se o apstrofo NO est na ltima posic
                //				{
                //					subStr=text.Split(separator2);
                //					text = subStr[0] + subStr[1];
                //				}
                //				else
                //				{
                //					text = Regex.Replace(text,"`","");
                //				}
                //				RemoveQuote(text);
            }
            //retirando "
            if (indice3 >= 0)
            {
                return text.Substring(0, indice3);
                //				if(text.Length-1>indice3 ) //se o apstrofo NO est na ltima posic
                //				{
                //					subStr=text.Split(separator3);
                //					text = subStr[0] + subStr[1];
                //				}
                //				else
                //				{
                //					text = Regex.Replace(text,"\"","");
                //				}
                //				RemoveQuote(text);

            }
            return text;
        }
    }
}